/*****************************************************************************/
/*  wtkbpp.cpp : Waveterm Keyboard simulation using the printer port         */
/*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <process.h>
#include <conio.h>
#include <io.h>

/*****************************************************************************/
/* Global Variables                                                          */
/*****************************************************************************/

unsigned short nBasePort = *((unsigned short far *)0x408);

#define DATA_PORT (nBasePort + 0)
#define STATUS_PORT (nBasePort + 1)
#define CONTROL_PORT (nBasePort + 2)

/*****************************************************************************/
/* Strobe : sets the STROBE flag either on or off                            */
/*****************************************************************************/

inline void Strobe(int bOn)
{
_outp(CONTROL_PORT, !bOn);  // set !STROBE flag
}

/*****************************************************************************/
/* GetTick : returns the current tick count                                  */
/*****************************************************************************/

inline int GetTick()
{
return *((unsigned short far *)0x46c);
}

/*****************************************************************************/
/* WaitTick : waits specified number of ticks (1st one may be shorter)       */
/*****************************************************************************/

inline void WaitTick(int nTicks = 1)
{
while (nTicks)
  {
  int volatile nCurTick = GetTick();
  while (nCurTick == GetTick())
    ;
  nTicks--;
  }
}

/*****************************************************************************/
/* SimKey : simulates a key                                                  */
/*****************************************************************************/

void SimKey(unsigned char cByte)
{
WaitTick();
_outp(DATA_PORT, cByte);
Strobe(1);

WaitTick();

Strobe(0);  
}

/*****************************************************************************/
/* SimString : simulates a complete string                                   */
/*****************************************************************************/

inline void SimString(char *lpszTxt)
{
while (*lpszTxt)
  SimKey(*lpszTxt++);
}

/*****************************************************************************/
/* GetKey : retrieves a key from the PC keyboard                             */
/*****************************************************************************/

int GetKey()
{
int c;

if (((c = getch()) == 0) || (c == 0xe0))
  c = getch() << 8;

return c;
}

/*****************************************************************************/
/*  Help : displays a help screen                                            */
/*****************************************************************************/

void Help()
{
printf("\nSpecial functions:\n\n");
printf("F1 ... this help page\n");
printf("F3 ... exit to DOS\n");
}

/*****************************************************************************/
/* main : main program function                                              */
/*****************************************************************************/

int main(int argc, char *argv[])
{
printf("Using printer at %04x\n", nBasePort);

Strobe(0);     // initialize port

printf("Type along, press F1 for Help, F3 to end:\n");

int nKey;
while ((nKey = GetKey()) != 0x3d00)     /* repeat until F3                   */
  {
  switch (nKey)
    {
    case 0x3b00 :                       /* F1 pressed ?                      */
      Help();
      break;

    case 0x3d00 :                       /* F3 pressed ?                      */
      return 0;                         /* get outta here                    */

    default :                           /* any other key ?                   */
      if (nKey < 0xff)                  /* if normal one                     */
        {
        SimKey((unsigned char)nKey);    /* pass it to Waveterm               */
#ifdef _DEBUG
        putch(nKey);
        if (nKey == '\r')
          putch('\n');
#endif
        }
      else
        {
        printf("\nSpecial key %04x ignored\n", nKey);
        }
      break;
    }
  }
  
return 0;
}
